<#
.SYNOPSIS
	This script performs the installation or uninstallation of an application(s).
	# LICENSE #
	PowerShell App Deployment Toolkit - Provides a set of functions to perform common application deployment tasks on Windows. 
	Copyright (C) 2017 - Sean Lillis, Dan Cunningham, Muhammad Mashwani, Aman Motazedian.
	This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
	You should have received a copy of the GNU Lesser General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
.DESCRIPTION
	The script is provided as a template to perform an install or uninstall of an application(s).
	The script either performs an "Install" deployment type or an "Uninstall" deployment type.
	The install deployment type is broken down into 3 main sections/phases: Pre-Install, Install, and Post-Install.
	The script dot-sources the AppDeployToolkitMain.ps1 script which contains the logic and functions required to install or uninstall an application.
.PARAMETER DeploymentType
	The type of deployment to perform. Default is: Install.
.PARAMETER DeployMode
	Specifies whether the installation should be run in Interactive, Silent, or NonInteractive mode. Default is: Interactive. Options: Interactive = Shows dialogs, Silent = No dialogs, NonInteractive = Very silent, i.e. no blocking apps. NonInteractive mode is automatically set if it is detected that the process is not user interactive.
.PARAMETER AllowRebootPassThru
	Allows the 3010 return code (requires restart) to be passed back to the parent process (e.g. SCCM) if detected from an installation. If 3010 is passed back to SCCM, a reboot prompt will be triggered.
.PARAMETER TerminalServerMode
	Changes to "user install mode" and back to "user execute mode" for installing/uninstalling applications for Remote Destkop Session Hosts/Citrix servers.
.PARAMETER DisableLogging
	Disables logging to file for the script. Default is: $false.
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -DeployMode 'Silent'; Exit $LastExitCode }"
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -AllowRebootPassThru; Exit $LastExitCode }"
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -DeploymentType 'Uninstall'; Exit $LastExitCode }"
.EXAMPLE
    Deploy-Application.exe -DeploymentType "Install" -DeployMode "Silent"
.NOTES
	Toolkit Exit Code Ranges:
	60000 - 68999: Reserved for built-in exit codes in Deploy-Application.ps1, Deploy-Application.exe, and AppDeployToolkitMain.ps1
	69000 - 69999: Recommended for user customized exit codes in Deploy-Application.ps1
	70000 - 79999: Recommended for user customized exit codes in AppDeployToolkitExtensions.ps1
.LINK 
	http://psappdeploytoolkit.com
#>
[CmdletBinding()]
Param (
    [Parameter(Mandatory = $false)]
    [ValidateSet('Install', 'Uninstall')]
    [string]$DeploymentType = 'Install',
    [Parameter(Mandatory = $false)]
    [ValidateSet('Interactive', 'Silent', 'NonInteractive')]
    [string]$DeployMode = 'Interactive',
    [Parameter(Mandatory = $false)]
    [switch]$AllowRebootPassThru = $false,
    [Parameter(Mandatory = $false)]
    [switch]$TerminalServerMode = $false,
    [Parameter(Mandatory = $false)]
    [switch]$DisableLogging = $false
)

Try {
    ## Set the script execution policy for this process
    Try { Set-ExecutionPolicy -ExecutionPolicy 'ByPass' -Scope 'Process' -Force -ErrorAction 'Stop' } Catch {}
	
    ##*===============================================
    ##* VARIABLE DECLARATION
    ##*===============================================
    ## Variables: Application
    [string]$appVendor = 'Cisco'
    [string]$appName = 'AnyConnect_Registry_Cleanup'
    [string]$appVersion = '4.7.01076'
    [string]$appArch = ''
    [string]$appTarget = 'ENT'	
    [string]$appLang = 'ENG'
    [string]$appRevision = 'A'
    [string]$appScriptVersion = '1.0.0'
    [string]$appScriptDate = '03/24/2020'
    [string]$appScriptAuthor = 'Lee Emerson'
    [string]$ESDNumber = '5851'
	[string]$ESDRequestor = 'Sivasubramanian Sudalaiyandi'
	[BOOL]$RebootRequired = $true #-$true or $false
	
	## Keep notes below on what changes were made if $appRevision changes
	# Version B - 
    
    [string]$StartTime = Get-Date -Format g
    [string]$Status = ''
    [string]$FINALMSG = ''
    ##*===============================================
    ## Variables: Install Titles (Only set here to override defaults set by the toolkit)
    [string]$installName = ''
    [string]$installTitle = 'Anyconnect Registry Cleanup'
	
    ##* Do not modify section below
    #region DoNotModify
	
    ## Variables: Exit Code
    [int32]$mainExitCode = 0
	
    ## Variables: Script
    [string]$deployAppScriptFriendlyName = 'Deploy Application'
    [version]$deployAppScriptVersion = [version]'3.7.0'
    [string]$deployAppScriptDate = '02/13/2018'
    [hashtable]$deployAppScriptParameters = $psBoundParameters
	
    ## Variables: Environment
    If (Test-Path -LiteralPath 'variable:HostInvocation') { $InvocationInfo = $HostInvocation } Else { $InvocationInfo = $MyInvocation }
    [string]$scriptDirectory = Split-Path -Path $InvocationInfo.MyCommand.Definition -Parent
	
    ## Dot source the required App Deploy Toolkit Functions
    Try {
        [string]$moduleAppDeployToolkitMain = "$scriptDirectory\AppDeployToolkit\AppDeployToolkitMain.ps1"
        If (-not (Test-Path -LiteralPath $moduleAppDeployToolkitMain -PathType 'Leaf')) { Throw "Module does not exist at the specified location [$moduleAppDeployToolkitMain]." }
        If ($DisableLogging) { . $moduleAppDeployToolkitMain -DisableLogging } Else { . $moduleAppDeployToolkitMain }
    }
    Catch {
        If ($mainExitCode -eq 0) { [int32]$mainExitCode = 60008 }
        Write-Error -Message "Module [$moduleAppDeployToolkitMain] failed to load: `n$($_.Exception.Message)`n `n$($_.InvocationInfo.PositionMessage)" -ErrorAction 'Continue'
        ## Exit the script, returning the exit code to SCCM
        If (Test-Path -LiteralPath 'variable:HostInvocation') { $script:ExitCode = $mainExitCode; Exit } Else { Exit $mainExitCode }
    }
	
    #endregion
    ##* Do not modify section above
    [string]$Attempts = Create-HWCustomRegistryFields
    ##*===============================================
    ##* END VARIABLE DECLARATION
    ##*===============================================
		
    If ($deploymentType -ine 'Uninstall') {
        ##*===============================================
        ##* PRE-INSTALLATION
        ##*===============================================
        [string]$installPhase = 'Uninstallation'
        
        [string]$configDeferPromptExpiryMessage = $configDeferPromptExpiryMessage.Replace('installation', 'uninstallation')
        [string]$configClosePromptMessage = $configClosePromptMessage.Replace('installation', 'uninstallation')
        [string]$configDiskSpaceMessage = $configDiskSpaceMessage.Replace('installation', 'uninstallation')
        [string]$configRestartPromptMessage = $configRestartPromptMessage.Replace('installation', 'uninstallation')
        [string]$configBlockExecutionMessage = $configBlockExecutionMessage.Replace('installation', 'uninstallation')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('IT will be reaching out to you to complete license registration', '')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('and can be used after installation.', '')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('installed', 'uninstalled')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('installation', 'uninstallation')
        
        Show-InstallationProgress -StatusMessage 'Uninstalling AnyConnect'
		
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "UNINSTALL NAM" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        Remove-MSIApplications -Name "Cisco AnyConnect Network Access Manager"
        
        if ($is64Bit -eq $true)
        {
            ##If path is to Program Files (x86) use this. Delete the other
            $WantFile = "$envProgramFilesX86\Cisco\Cisco AnyConnect Secure Mobility Client\acnamagent.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                    Write-Log -Message "NAM UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "NAM UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        else
        {
            $WantFile = "$envProgramFiles\Cisco\Cisco AnyConnect Secure Mobility Client\acnamagent.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                    Write-Log -Message "NAM UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile $NewFileVersion" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "NAM UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                
                Exit-Script -ExitCode 1
            }
        }
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "UNINSTALL DART" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Remove-MSIApplications -Name "Cisco AnyConnect Diagnostics and Reporting Tool"
        
        if ($is64Bit -eq $true)
        {
            ##If path is to Program Files (x86) use this. Delete the other
            $WantFile = "$envProgramFilesX86\Cisco\Cisco AnyConnect Secure Mobility Client\Dart\dartcli.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                    Write-Log -Message "DART UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile $NewFileVersion" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "DART UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        else
        {
            $WantFile = "$envProgramFiles\Cisco\Cisco AnyConnect Secure Mobility Client\Dart\dartcli.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                Write-Log -Message "DART UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile $NewFileVersion" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "DART UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "UNINSTALL VPN" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Remove-MSIApplications -Name "Cisco AnyConnect Secure Mobility Client"
        
        if ($is64Bit -eq $true)
        {
            ##If path is to Program Files (x86) use this. Delete the other
            $WantFile = "$envProgramFilesX86\Cisco\Cisco AnyConnect Secure Mobility Client\vpnui.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                Write-Log -Message "VPN UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "VPN UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        else
        {
            $WantFile = "$envProgramFiles\Cisco\Cisco AnyConnect Secure Mobility Client\vpnui.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                Write-Log -Message "VPN UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile $NewFileVersion" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "VPN UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "UNINSTALL UMBRELLA" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Remove-MSIApplications -Name "Cisco AnyConnect Umbrella Roaming Security Module "
        
        if ($is64Bit -eq $true)
        {
            ##If path is to Program Files (x86) use this. Delete the other
            $WantFile = "$envProgramFilesX86\Cisco\Cisco AnyConnect Secure Mobility Client\UmbrellaDiagnostic.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                Write-Log -Message "UMBRELLA UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "UMBRELLA UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        else
        {
            $WantFile = "$envProgramFiles\Cisco\Cisco AnyConnect Secure Mobility Client\UmbrellaDiagnostic.exe"
            $version_we_require = "$UPDATE_VERSION"
            $FileExists = Test-Path $WantFile
            If ($FileExists -eq $false)
            {
                Write-Log -Message "UMBRELLA UNINSTALL SUCCESSFUL. FILE NOT FOUND: $WantFile $NewFileVersion" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "UMBRELLA UNINSTALL FAILED. FILE STILL FOUND: $WantFile"
                $Status = "Failed"
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Show-InstallationPrompt -Message "$FINALMSG" -ButtonRightText 'OK' -Icon Information -NoWait
                Exit-Script -ExitCode 1
            }
        }
        
		
        ##*===============================================
        ##* INSTALLATION 
        ##*===============================================
        [string]$installPhase = 'Installation'
		
        ## Handle Zero-Config MSI Installations
        If ($useDefaultMsi) {
            [hashtable]$ExecuteDefaultMSISplat = @{ Action = 'Install'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
            Execute-MSI @ExecuteDefaultMSISplat; If ($defaultMspFiles) { $defaultMspFiles | ForEach-Object { Execute-MSI -Action 'Patch' -Path $_ } }
        }
		
       	Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "RUN PURGENOTIFYOBJECTS.EXE " -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		
        Show-InstallationProgress       
        if ($is64Bit -eq $true)
        {
            Execute-Process -Path "$dirFiles\amd64\PurgeNotifyObjects.exe" -Parameters '-confirmdelete' -WindowStyle 'Hidden'
        	
        }
        Else
        {
            Execute-Process -Path "$dirFiles\x86\PurgeNotifyObjects.exe" -Parameters '-confirmdelete' -WindowStyle 'Hidden'
        }
		

    
        ##*===============================================
        ##* POST-INSTALLATION
        ##*===============================================
        [string]$installPhase = 'Post-Installation'
		
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Update-HWCustomRegistryFields -Status "Success" -FINALMSG "Successful Installation" -ExitCode $returnCode
       
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "RUN HW INVENTORY POLICY" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Invoke-SCCMTask -ScheduleID 'HardwareInventory'
        #Invoke-SCCMTask -ScheduleID 'ApplicationManagerPolicyAction'
        Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		
        If ($RebootRequired) {Show-InstallationRestartPrompt -NoCountdown}
        #Show-InstallationRestartPrompt -Countdownseconds 28800 -CountdownNoHideSeconds 60
        ## Display a message at the end of the install
        #If (-not $useDefaultMsi) { Show-InstallationPrompt -Message "$installTitle installed successfully" -ButtonRightText 'OK' -Icon Information -NoWait }
    }
    ElseIf ($deploymentType -ieq 'Uninstall') {
        ##*===============================================
        ##* PRE-UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Pre-Uninstallation'
        [string]$configDeferPromptExpiryMessage = $configDeferPromptExpiryMessage.Replace('installation', 'uninstallation')
        [string]$configClosePromptMessage = $configClosePromptMessage.Replace('installation', 'uninstallation')
        [string]$configDiskSpaceMessage = $configDiskSpaceMessage.Replace('installation', 'uninstallation')
        [string]$configRestartPromptMessage = $configRestartPromptMessage.Replace('installation', 'uninstallation')
        [string]$configBlockExecutionMessage = $configBlockExecutionMessage.Replace('installation', 'uninstallation')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('IT will be reaching out to you to complete license registration', '')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('and can be used after installation.', '')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('installed', 'uninstalled')
        [string]$configDeferPromptWelcomeMessage = $configDeferPromptWelcomeMessage.Replace('installation', 'uninstallation')
		
        ## Show Welcome Message, close Internet Explorer with a 60 second countdown before automatically closing
		Show-InstallationWelcome -CloseApps 'iexplore' -CloseAppsCountdown 60
		#Show-InstallationWelcome
		
        ## Show Progress Message (with the default message)
        Show-InstallationProgress
		
        ## <Perform Pre-Uninstallation tasks here>
		
				
        ##*===============================================
        ##* UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Uninstallation'
		
        ## Handle Zero-Config MSI Uninstallations
        If ($useDefaultMsi) {
            [hashtable]$ExecuteDefaultMSISplat = @{ Action = 'Uninstall'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
            Execute-MSI @ExecuteDefaultMSISplat
        }
		
		# <Perform Uninstallation tasks here>
		Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		Write-Log -Message "UNINSTALL $installTitle" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		
		##  <Example of MSI uninstall>
		##  <Name as displayed in Programs & Features or in Displayname in uninstall reg key>
		##  Remove-MSIApplications -Name "Software Name"
		
		
		Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		Write-Log -Message "VERIFY SUCCESSFUL UNINSTALL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		
		##  <Get one of the methods from the cheat sheet #region UNINSTALL - VERIFY SUCCESSFUL UNINSTALL to verify a successful install>
		
		
		##*===============================================
        ##* POST-UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Post-Uninstallation'
		
        ## <Perform Post-Uninstallation tasks here>
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Remove-HWCustomRegistryFields
        Update-HWCustomRegistryFields -Status "Success" -FINALMSG "Successful Uninstall" -ExitCode $returnCode
		
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "RUN HW INVENTORY POLICY" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Invoke-SCCMTask -ScheduleID 'HardwareInventory'
        #Invoke-SCCMTask -ScheduleID 'ApplicationManagerPolicyAction'
        Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
		
        If ($RebootRequired) {Show-InstallationRestartPrompt -NoCountdown}
		#Show-InstallationRestartPrompt -Countdownseconds 28800 -CountdownNoHideSeconds 60
        #Show-InstallationPrompt -Message "$installTitle uninstalled successfully"  -ButtonRightText 'OK' -Icon Information -NoWait
    }

		
	
	
    ##*===============================================
    ##* END SCRIPT BODY
    ##*===============================================
	
    ## Call the Exit-Script function to perform final cleanup operations
    Exit-Script -ExitCode $mainExitCode
}
Catch {
    
    Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
    Write-Log -Message "AN ERROR WAS DETECTED" -LogType 'CMTrace' -Severity 3 -Source $deployAppScriptFriendlyName
    Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName

    [int32]$mainExitCode = 60001
    [string]$mainErrorMessage = "$(Resolve-Error)"
    Write-Log -Message $mainErrorMessage -Severity 3 -Source $deployAppScriptFriendlyName
    Show-DialogBox -Text $mainErrorMessage -Icon 'Stop'
    Exit-Script -ExitCode $mainExitCode
}
# SIG # Begin signature block
# MIIRFwYJKoZIhvcNAQcCoIIRCDCCEQQCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUafLwZ8lPEkYabiCXJjSzaLB1
# 6E2ggg5ZMIIDtzCCAp+gAwIBAgIQDOfg5RfYRv6P5WD8G/AwOTANBgkqhkiG9w0B
# AQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMzExMTEwMDAwMDAwWjBlMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3Qg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCtDhXO5EOAXLGH87dg
# +XESpa7cJpSIqvTO9SA5KFhgDPiA2qkVlTJhPLWxKISKityfCgyDF3qPkKyK53lT
# XDGEKvYPmDI2dsze3Tyoou9q+yHyUmHfnyDXH+Kx2f4YZNISW1/5WBg1vEfNoTb5
# a3/UsDg+wRvDjDPZ2C8Y/igPs6eD1sNuRMBhNZYW/lmci3Zt1/GiSw0r/wty2p5g
# 0I6QNcZ4VYcgoc/lbQrISXwxmDNsIumH0DJaoroTghHtORedmTpyoeb6pNnVFzF1
# roV9Iq4/AUaG9ih5yLHa5FcXxH4cDrC0kqZWs72yl+2qp/C3xag/lRbQ/6GW6whf
# GHdPAgMBAAGjYzBhMA4GA1UdDwEB/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB0G
# A1UdDgQWBBRF66Kv9JLLgjEtUYunpyGd823IDzAfBgNVHSMEGDAWgBRF66Kv9JLL
# gjEtUYunpyGd823IDzANBgkqhkiG9w0BAQUFAAOCAQEAog683+Lt8ONyc3pklL/3
# cmbYMuRCdWKuh+vy1dneVrOfzM4UKLkNl2BcEkxY5NM9g0lFWJc1aRqoR+pWxnmr
# EthngYTffwk8lOa4JiwgvT2zKIn3X/8i4peEH+ll74fg38FnSbNd67IJKusm7Xi+
# fT8r87cmNW1fiQG2SVufAQWbqz0lwcy2f8Lxb4bG+mRo64EtlOtCt/qMHt1i8b5Q
# Z7dsvfPxH2sMNgcWfzd8qVttevESRmCD1ycEvkvOl77DZypoEd+A5wwzZr8TDRRu
# 838fYxAe+o0bJW1sj6W3YQGx0qMmoRBxna3iw/nDmVG3KwcIzi7mULKn+gpFL6Lw
# 8jCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcNAQELBQAw
# ZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQ
# d3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBS
# b290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjELMAkGA1UE
# BhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2lj
# ZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUg
# U2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPjTsxx/
# DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5idQ3Gde2
# qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQdLEoJrsk
# acLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+rHCiq85/
# 6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE86xnTrXE
# 94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pzINAPZHM8
# np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8CAQAwDgYD
# VR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUFBwEBBG0w
# azAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUF
# BzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVk
# SURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsNC5kaWdp
# Y2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRw
# Oi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3Js
# ME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczov
# L3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRaxLl7
# KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823I
# DzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwtOhrE7zBh
# 134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GOYw4TS63X
# X0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEoAjwukaPA
# JRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhCMrI2iiQC
# /i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOBm0/GkxAG
# /AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zCCBWYwggROoAMC
# AQICEAREyrrPH/bIC6a8qkAADdUwDQYJKoZIhvcNAQELBQAwcjELMAkGA1UEBhMC
# VVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0
# LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2ln
# bmluZyBDQTAeFw0yMTAzMjYwMDAwMDBaFw0yNDA0MDMyMzU5NTlaMIGjMQswCQYD
# VQQGEwJVUzETMBEGA1UECBMKTmV3IEplcnNleTEWMBQGA1UEBxMNTW9ycmlzIFBs
# YWluczElMCMGA1UEChMcSG9uZXl3ZWxsIEludGVybmF0aW9uYWwgSW5jLjEZMBcG
# A1UECxMQRW5kIFVzZXIgQ29tcHV0ZTElMCMGA1UEAxMcSG9uZXl3ZWxsIEludGVy
# bmF0aW9uYWwgSW5jLjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJec
# P0ZxP4vDmqGIqqw7KswHwDOQWqDeWQcqpSiuwQmClTB1R7VgkQMDancIHcSPBZ25
# 1dppCuUgpbSmfTOZ+pG0I9oJY2rZOa6pf3BgHfuQSmSD/gDBmZ8XzGsIx/Vck9Ij
# 79YBqL7Gr6as5c0NiYP2mg9fDLbo0yDKQKvl26SdwEz/QsfNwMWVEnXj3Uv51z+W
# DtExIqr0XHIYwIoR2hwQ6iLERgBGFs1bUCFo7mWUtCkK7237lptHajeeFzD6tROE
# dhYpVvP9Y+LUcwlJ8+NmsX0s9E4puCRIweaERVLituHgfAHmzh/7SxbSd7UixGBq
# 3RwgdP3L/hIrSLXdBxECAwEAAaOCAcQwggHAMB8GA1UdIwQYMBaAFFrEuXsqCqOl
# 6nEDwGD5LfZldQ5YMB0GA1UdDgQWBBTrJDkqG5e5J8vh+W0fVSplLA2HJDAOBgNV
# HQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwdwYDVR0fBHAwbjA1oDOg
# MYYvaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5j
# cmwwNaAzoDGGL2h0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9zaGEyLWFzc3VyZWQt
# Y3MtZzEuY3JsMEsGA1UdIAREMEIwNgYJYIZIAYb9bAMBMCkwJwYIKwYBBQUHAgEW
# G2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUF
# BwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4G
# CCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRT
# SEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkq
# hkiG9w0BAQsFAAOCAQEAODhorvbDyk1CeAv6BXsSdcuQlIMDKr4/RVf87AIS6eQZ
# 0x8bzVZ64ndKUEs7vgHfkIy3IITGmtGk4w2hvv/O3P4YutkxTY4m3Oh6DOCRjXUu
# pkoRAMjPf4CzQ0aXONTGCBazuWf7GXNVsE7Qdt1JC3BeDNXhcLGbeb8bZQwu/RP6
# 3JLwvttknttcCPoWnH9IEdpW4AT+53xKaAeL4yF0x5XXzd8L05dXEuReYnw6ED3X
# 1mHa++xVSU+SxC19cA6DZiOSXVvTp8z/XyTc160oc4bfl8DFoU3zgx7szwn6eAwq
# sPlXQyryAL/QBVCzRh3Yf43wHdXZx8doD9qReVFuzDGCAigwggIkAgEBMIGGMHIx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJ
# RCBDb2RlIFNpZ25pbmcgQ0ECEAREyrrPH/bIC6a8qkAADdUwCQYFKw4DAhoFAKB4
# MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQB
# gjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkE
# MRYEFNMejewozCRBc+xSEQW1bbB7re8/MA0GCSqGSIb3DQEBAQUABIIBAExbSajB
# 0cu1uBBV2SW2QQwjQNR7LmlMZV1BqNoe3Ywn4fUrEB5SMBdYQEDpqictrxeAFlRc
# 8g8iM+cuDgMAgzUOprwX/6AjWkmz7aM+d/ZGOVeyv4wRkzwdPq0JcOelEfIFJYK4
# Inv+vrwhe3TEYhIQhoqvmbmosb4Fn/0eSYgz9Guun0dzyDRHKpuiIdFPFk5cmdBU
# 64nOzmRaC3cIWuUlItQ1AWCwdktyQB1q/7m78vLSyVWxjNykr5mwBzTFVfwToDBE
# tftP/dN9v5attQtgvqB+dQtg1NlEGMcAVk/lEDAakXukHgqlrXRjhK4jkdVHXBWi
# mhN66lwf3mV/It8=
# SIG # End signature block
